package de.szut.lf8_starter.integration.project;

import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectRepository;
import org.json.JSONObject;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
class UpdateProjectActionTest {
    @Autowired
    private MockMvc mockMvc;
    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void updateProjectShouldUpdateProject() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setId(1);
        project.setComment("comment");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setEndDate(LocalDate.of(2024, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("name");
        project.setStartDate(LocalDate.of(2021, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);

        String content = """
                {
                    "name": "updatedName",
                    "leading_employee": 2,
                    "employees": [3, 4, 5],
                    "contractor": 6,
                    "contractor_name": "Updated Contractor name",
                    "comment": "new goal of project",
                    "start_date": "01.01.2021",
                    "planned_end_date": "01.01.2022"
                  }
                """;

        final var contentAsString = this.mockMvc.perform(
                        put("/projects/1").content(content).contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andExpect(jsonPath("name", is("updatedName")))
                .andExpect(jsonPath("leading_employee", is(2)))
                .andExpect(jsonPath("employees", is(Arrays.asList(3, 4, 5))))
                .andExpect(jsonPath("contractor", is(6)))
                .andExpect(jsonPath("contractor_name", is("Updated Contractor name")))
                .andExpect(jsonPath("comment", is("new goal of project")))
                .andExpect(jsonPath("start_date", is("01.01.2021")))
                .andExpect(jsonPath("planned_end_date", is("01.01.2022")))
                .andReturn()
                .getResponse()
                .getContentAsString();

        final var id = Long.parseLong(new JSONObject(contentAsString).get("id").toString());

        final var existingProject = this.projectRepository.findById(id);
        assertThat(existingProject.get().getName()).isEqualTo("updatedName");
        assertThat(existingProject.get().getLeadingEmployee()).isEqualTo(2);
        assertThat(existingProject.get().getContractor()).isEqualTo(6);
        assertThat(existingProject.get().getContractorName()).isEqualTo("Updated Contractor name");
        assertThat(existingProject.get().getComment()).isEqualTo("new goal of project");
        assertThat(existingProject.get().getStartDate()).isEqualTo(LocalDate.of(2021, 1, 1));
        assertThat(existingProject.get().getPlannedEndDate()).isEqualTo(LocalDate.of(2022, 1, 1));

    }

    @Test
    void updateProjectShouldUpdateProjectPartially() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setId(1);
        project.setName("name");
        project.setLeadingEmployee(1);
        project.setContractor(1);
        project.setComment("comment");
        project.setEmployees(List.of(1L, 2L, 3L));
        project.setContractorName("contractorName");
        project.setStartDate(LocalDate.of(2021, 1, 1));
        project.setPlannedEndDate(LocalDate.of(2023, 1, 1));
        project.setEndDate(LocalDate.of(2024, 1, 1));
        this.projectRepository.save(project);

        String content = """
                {}
                """;

        final var contentAsString = this.mockMvc.perform(
                        put("/projects/1").content(content).contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andExpect(jsonPath("name", is("name")))
                .andExpect(jsonPath("leading_employee", is(1)))
                .andExpect(jsonPath("employees", is(List.of(1,2,3))))
                .andExpect(jsonPath("contractor", is(1)))
                .andExpect(jsonPath("contractor_name", is("contractorName")))
                .andExpect(jsonPath("comment", is("comment")))
                .andExpect(jsonPath("start_date", is("01.01.2021")))
                .andExpect(jsonPath("planned_end_date", is("01.01.2023")))
                .andExpect(jsonPath("end_date", is("01.01.2024")))
                .andReturn()
                .getResponse()
                .getContentAsString();

        final var id = Long.parseLong(new JSONObject(contentAsString).get("id").toString());

        final var existingProject = this.projectRepository.findById(id);
        assertThat(existingProject.get().getName()).isEqualTo("name");
        assertThat(existingProject.get().getLeadingEmployee()).isEqualTo(1);
        assertThat(existingProject.get().getContractor()).isEqualTo(1);
        assertThat(existingProject.get().getContractorName()).isEqualTo("contractorName");
        assertThat(existingProject.get().getComment()).isEqualTo("comment");
        assertThat(existingProject.get().getStartDate()).isEqualTo(LocalDate.of(2021, 1, 1));
        assertThat(existingProject.get().getPlannedEndDate()).isEqualTo(LocalDate.of(2023, 1, 1));
        assertThat(existingProject.get().getEndDate()).isEqualTo(LocalDate.of(2024, 1, 1));
    }

    @Test
    void updateProjectShouldReturnNotFoundResponseWhenProjectIsNotFound() throws Exception {
        this.mockMvc.perform(put("/projects/2").content("{}").contentType(MediaType.APPLICATION_JSON)).andExpect(status().isNotFound());
    }
}
